/* NOSCOREHOGS limits each user to one high score in the global list */
#undef NOSCOREHOGS

/* USE_REAL_NAMES enters user names from the system's user list instead of asking */
#undef USE_REAL_NAMES

/* Whether to compile with sound support or not */
#undef SOUND

/* Use NAS instead of a native sound driver */
#undef NAS_SOUND

/* The sound device to use (E.G. /dev/dsp) */
#undef SOUNDDEV

/* Where to put the global high scores */
#undef SCOREFILE

/* Where to put the sound files */
#undef SOUNDDIR

/* ----------------------------------------------------
 * You shouldn't have to touch anything below this line
 * ----------------------------------------------------
 */
#undef IMAGEDIR
@TOP@
